@@ -462,15 +462,6 @@ void safexcel_dequeue(struct safexcel_crypto_priv *priv, int ring)
 		if (backlog)
 			backlog->complete(backlog, -EINPROGRESS);
 
-		/* In case the send() helper did not issue any command to push
-		 * to the engine because the input data was cached, continue to
-		 * dequeue other requests as this is valid and not an error.
-		 */
-		if (!commands && !results) {
-			kfree(request);
-			continue;
-		}
-
 		spin_lock_bh(&priv->ring[ring].egress_lock);
 		list_add_tail(&request->list, &priv->ring[ring].list);
 		spin_unlock_bh(&priv->ring[ring].egress_lock);
@@ -798,7 +789,7 @@ static int safexcel_probe(struct platform_device *pdev)
 		return PTR_ERR(priv->base);
 	}
 
-	priv->clk = devm_clk_get(&pdev->dev, NULL);
+	priv->clk = of_clk_get(dev->of_node, 0);
 	if (!IS_ERR(priv->clk)) {
 		ret = clk_prepare_enable(priv->clk);
 		if (ret) {
